# -*- encoding: utf-8 -*-

import multiprocessing
import os
import platform

from suite_py.lib import logger
from suite_py.lib.config import Config
from suite_py.lib.handler.captainhook_handler import CaptainHook


# Collects metrics and sends them to datadog through captainhook
class Metrics:
    def __init__(self, config: Config):
        self._config = config

    # Emits the command_executed metric
    def command_executed(self, command: str, success: None | bool = True):
        metric_data = {
            "type": "command_executed",
            "os": platform.system(),
            "command": command,
            "success": success,
        }

        self._create_metric(metric_data)

    def _create_metric(self, metric):
        # Instead of trying to submit metrics all at once first we save them to a file first
        # That way we can batch metric submissions and retry them if they fail (eg. because the user isn't authenticated with okta)
        metrics = self._config.get_cookie("metrics", [])
        if not isinstance(metrics, list):
            logger.warning(
                f"Metrics cookie is not a list! Replacing {metrics} with an empty list"
            )
            metrics = []

        logger.debug(f"creating metric: {metric}")
        metrics.append(metric)
        self._config.put_cookie("metrics", metrics)

    # Upload metrics in a detached background process
    def async_upload(self):
        # Double fork to detach the process
        def child():
            if (
                # Windows doesn't support forking.
                # There aren't any windows users that we know of so we can just block application exit
                # If windows users appear we should fix this
                os.name == "posix"
                and
                # Forking here is safe since we are running in a fresh process spawned by multiprocessing
                os.fork() != 0
            ):
                return

            self.upload()

        p = multiprocessing.Process(target=child)
        p.start()
        p.join()

    def upload(self):
        captainhook = CaptainHook(self._config)

        metrics = self._config.get_cookie("metrics", [])
        # Prevent double uploads
        #
        # This isn't atomic and could still lead to the same metrics being uploaded twice
        # but this occuring is unlikely enough that it shouldn't significantly influence our stats
        self._config.put_cookie("metrics", [])
        try:
            if len(metrics) != 0:
                captainhook.send_metrics(metrics)
        except Exception:
            # Upload failed, try again later
            self._config.put_cookie("metrics", metrics)



# -*- coding: utf-8 -*-
import sys

import requests

from suite_py.lib import logger
from suite_py.lib import metrics
from suite_py.lib.handler.captainhook_handler import CaptainHook


class ProjectLock:
    def __init__(self, project, env, action, config, tokens):
        self._project = project
        self._env = _parse_env(env)
        self._action = action
        self._captainhook = CaptainHook(config, tokens=tokens)

    @metrics.command("manage-project-lock")
    def run(self):
        if self._action == "lock":
            try:
                req = self._captainhook.lock_project(self._project, self._env)
                _handle_request(req)
                logger.info(f"Locked deploy on {self._env} of {self._project} project")
            except requests.exceptions.Timeout:
                logger.warning(
                    "Captainhook request timed out. Try with suite-py --timeout=60 lock-project lock"
                )
                sys.exit(1)
        elif self._action == "unlock":
            try:
                req = self._captainhook.unlock_project(self._project, self._env)
                _handle_request(req)
                logger.info(
                    f"Unlocked deploy on {self._env} of {self._project} project"
                )
            except requests.exceptions.Timeout:
                logger.warning(
                    "Captainhook request timed out. Try with suite-py --timeout=60 lock-project lock"
                )
                sys.exit(1)
        else:
            logger.warning("I'm confused. Make the correct choice.")
            sys.exit(-1)


def _handle_request(request):
    if request.status_code != 200:
        logger.error(
            "Something went wrong during the request. Request DevOps support on Slack."
        )
        sys.exit(-1)

    return True


def _parse_env(env):
    # implementare uno switch era troppo noioso
    if env == "deploy":
        return "production"
    if env == "merge":
        return "staging"
    return env



# -*- encoding: utf-8 -*-
import logging
import logzero
from logzero import logger as _logger

DEFAULT_FORMAT = "%(color)s[%(levelname)1.4s]%(end_color)s %(message)s"
# Set a custom formatter
formatter = logzero.LogFormatter(fmt=DEFAULT_FORMAT)


def setup(verbose):
    logzero.setup_default_logger(formatter=formatter)
    level = logging.DEBUG if verbose else logging.INFO

    _logger.setLevel(level)
    _logger.debug("Logging as %s", level)


def debug(message, *args, **kwargs):
    _logger.debug(message, *args, **kwargs)


def info(message, *args, **kwargs):
    _logger.info(message, *args, **kwargs)


def warning(message, *args, **kwargs):
    _logger.warning(message, *args, **kwargs)


def error(message, *args, **kwargs):
    _logger.error(message, *args, **kwargs)



# -*- coding: utf-8 -*-
import re
import sys

from halo import Halo
from kubernetes import client, config
from rich.console import Console
from rich.table import Table

from suite_py.lib import logger
from suite_py.lib import metrics
from suite_py.lib.handler import prompt_utils
from suite_py.lib.handler.captainhook_handler import CaptainHook


class Aggregator:
    def __init__(self, config, command):
        self._captainhook = CaptainHook(config)
        self._command = command

    @metrics.command("aggregator")
    def run(self):
        if self._command == "list":
            self._list_aggregators()
            return

        if self._command == "change":
            aggregator = self._select_aggregator()
            address = prompt_utils.ask_questions_input(
                "Insert QA url or press enter to set staging URL: ",
                default_text="staging.prima.it",
            )

            if address.startswith("http"):
                address = re.sub(r"https?://?", "", address)

            split_addr = re.split(r"([a-z]+)\-([a-z0-9]+)", address)

            change_request = self._captainhook.change_aggregator(
                aggregator["name"], address
            )

            self._handle_captainhook_response(
                change_request, aggregator["name"], address
            )

            if "www" in split_addr:
                split_addr[1] = "prima"
                self._update_k8s_ingress_route(aggregator, split_addr[1], split_addr[2])
            elif "evvivass" in split_addr:
                self._update_k8s_ingress_route(aggregator, split_addr[1], split_addr[2])
            else:
                return

    def _handle_captainhook_response(self, request, aggregator, address):
        if request.status_code == 200:
            change_request = request.json()
            if change_request["success"]:
                logger.info(f"CNAME updated! Now {aggregator} is pointing to {address}")
            else:
                cases = {
                    "cloudflare_error": "Error during Cloudflare invocation.",
                    "unknown_dns_record": "Impossible to find DNS record associated with aggregator.",
                    "unknown_aggregator": "Aggregator not found.",
                    "invalid_qa_address": "The QA address does not meet the requirements.",
                }
                logger.error(cases.get(change_request["error"], "unknown error"))
                sys.exit(-1)
        else:
            logger.error("An error has occurred on Captainhook.")
            sys.exit(-1)

    def _select_aggregator(self):
        with Halo(text="Loading aggregators...", spinner="dots", color="magenta"):
            choices = [
                {"name": agg["name"], "value": agg}
                for agg in self._captainhook.aggregators().json()
            ]
        if choices:
            choices.sort(key=lambda x: x["name"])
            return prompt_utils.ask_choices("Select aggregator: ", choices)

        logger.error("There are no aggregators on Captainhook.")
        sys.exit(-1)

    def _list_aggregators(self):
        with Halo(text="Loading...", spinner="dots", color="magenta"):
            aggregators = self._captainhook.aggregators()

        if aggregators.status_code != 200:
            logger.error("Unable to retrieve the list of aggregators.")
            return

        console = Console()

        aggregators_table = Table()
        aggregators_table.add_column("Name", style="green")
        aggregators_table.add_column("Address", style="white")

        for a in aggregators.json():
            aggregators_table.add_row(a["name"], a["content"])

        logger.info("Available aggregators:")
        console.print(aggregators_table)

    def _update_k8s_ingress_route(self, aggregator, service_name, namespace):
        try:
            config.load_kube_config()
        except Exception:
            logger.error(
                "\n\nYou need to authenticate, run the following command:\n$ aws eks update-kubeconfig --name main-qa\n\n"
            )
            sys.exit(-1)

        api = client.CustomObjectsApi()
        res = {
            "apiVersion": "traefik.containo.us/v1alpha1",
            "kind": "IngressRoute",
            "metadata": {
                "name": f"{aggregator['name']}",
                "namespace": f"{namespace}",
                "annotations": {
                    "traefik.ingress.kubernetes.io/router.tls": "true",
                },
                "labels": {
                    "com.prima.environment": "qa",
                    "com.prima.country": "it",
                    "app.kubernetes.io/name": f"{aggregator['name']}",
                },
            },
            "spec": {
                "entryPoints": ["websecure"],
                "routes": [
                    {
                        "kind": "Rule",
                        "match": f"Host(`{aggregator['address']}{aggregator['domain']}`)",
                        "services": [{"name": f"{service_name}", "port": 80}],
                    }
                ],
            },
        }

        try:
            api.create_namespaced_custom_object(
                group="traefik.containo.us",
                version="v1alpha1",
                namespace=namespace,
                plural="ingressroutes",
                body=res,
            )
            logger.info(f"Aggregator {aggregator['name']} created!")
        except Exception:
            api.patch_namespaced_custom_object(
                group="traefik.containo.us",
                version="v1alpha1",
                namespace=namespace,
                plural="ingressroutes",
                name=aggregator["name"],
                body=res,
            )
            logger.info(f"Aggregator {aggregator['name']} updated!")



import sys

from suite_py.lib import logger
from suite_py.lib import metrics
from suite_py.lib.handler import prompt_utils
from suite_py.lib.tokens import Tokens


class SetToken:
    def __init__(self, tokens: Tokens):
        self._tokens = tokens

    @metrics.command("set-token")
    def run(self):
        selected = self._select_token()
        if not selected or selected.lower() != selected or selected.strip() != selected:
            # We bail out here, rather than modifying the provided token name, to avoid confusion
            logger.info(
                f'"{selected}" is not a valid token name. Token names must be lowercase and have no trailing whitespace'
            )
            sys.exit(-1)
        new_value = prompt_utils.ask_questions_input(
            f"Insert the new value for {selected}:"
        )
        if not new_value:
            logger.info("Token value cannot be empty")
            sys.exit(-1)
        self._tokens.edit(selected, new_value)
        self._tokens.save()
        logger.info(f"Token {selected} updated")

    def _select_token(self):
        choices = list(self._tokens.keys()) + ["Other..."]
        selected = prompt_utils.ask_choices(
            "Which token do you want to set?", choices, choices[0]
        )

        return (
            self._prompt_custom_token()
            if selected == "Other..."
            else selected.split(" ")[0]
        )

    def _prompt_custom_token(self):
        return prompt_utils.ask_questions_input("Enter custom token name:")



# -*- encoding: utf-8 -*-
import sys

import requests

from suite_py.lib import logger
from suite_py.lib.handler.github_handler import GithubHandler


class CaptainHook:
    def __init__(self, config, tokens=None):
        self._baseurl = config.user["captainhook_url"]
        self._timeout = config.user["captainhook_timeout"]
        self._okta_base_url = config.okta["base_url"]
        self._okta_token = config.get_cache(f"{self._okta_base_url}_token")
        self._headers = {
            "Authorization": f"Bearer {self._okta_token}",
        }

        if tokens is not None:
            self._github = GithubHandler(tokens)

    def lock_project(self, project, env):
        data = {
            "project": project,
            "status": "locked",
            "user": self._get_user(),
            "environment": env,
        }
        return self.send_post_request("/projects/manage-lock", data)

    def unlock_project(self, project, env):
        data = {
            "project": project,
            "status": "unlocked",
            "user": self._get_user(),
            "environment": env,
        }
        return self.send_post_request("/projects/manage-lock", data)

    def status(self, project, env):
        return self.send_get_request(
            f"/projects/check?project={project}&environment={env}"
        )

    def check(self):
        return requests.get(f"{self._baseurl}/", timeout=(2, self._timeout))

    def get_users_list(self):
        return self.send_get_request("/users/all")

    def aggregators(self):
        return self.send_get_request("/cloudflare/aggregators/available")

    def change_aggregator(self, aggregator, qa_address):
        data = {"aggregator": aggregator, "qa_address": qa_address}
        return self.send_put_request("/cloudflare/aggregators", data)

    def send_metrics(self, metrics):
        self.send_post_request("/suite_py/metrics/", json=metrics).raise_for_status()

    def send_post_request(self, endpoint, data=None, json=None):
        try:
            r = requests.post(
                f"{self._baseurl}{endpoint}",
                headers=self._headers,
                data=data,
                json=json,
                timeout=self._timeout,
            )

            return self._response(r)
        except Exception as e:
            logger.error(
                "Unable to contact Captainhook, are you logged in/using the VPN?"
            )
            raise e

    def send_put_request(self, endpoint, data=None, json=None):
        try:
            r = requests.put(
                f"{self._baseurl}{endpoint}",
                headers=self._headers,
                data=data,
                json=json,
                timeout=self._timeout,
            )

            return self._response(r)
        except Exception as e:
            logger.error(
                "Unable to contact Captainhook, are you logged in/using the VPN?"
            )
            raise e

    def send_get_request(self, endpoint):
        try:
            r = requests.get(
                f"{self._baseurl}{endpoint}",
                headers=self._headers,
                timeout=(2, self._timeout),
            )

            return self._response(r)
        except Exception as e:
            logger.error(
                "Unable to contact Captainhook, are you logged in/using the VPN?"
            )
            raise e

    def _response(self, response):
        if response.status_code == 401:
            logger.error("Unauthorized. Are you logged in?")
            sys.exit(-1)

        return response

    def set_timeout(self, timeout):
        self._timeout = timeout

    def _get_user(self):
        return self._github.get_user().login
